/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.core.common.notification.ISubscriptionFactory;
import org.opengroup.osdu.core.common.notification.ISubscriptionService;
import org.opengroup.osdu.core.common.notification.SubscriptionException;
import org.opengroup.osdu.core.common.notification.SubscriptionService;
import org.opengroup.osdu.notification.di.SubscriptionCacheFactory;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.fail;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class SubscriptionHandlerTest {

    private static final String NOTIFICATION_ID = "test-notification-id";

    @Mock
    private ISubscriptionFactory subscriptionFactory;

    @Mock
    private SubscriptionCacheFactory subscriptionCacheFactory;

    @Mock
    private ObjectMapper objectMapper;

    @InjectMocks
    private SubscriptionHandler sut;

    @Test(expected = AppException.class)
    public void should_throwException_whenSubscriptionNotFound() throws Exception {
        when(this.subscriptionCacheFactory.get(any())).thenReturn(null);
        ISubscriptionService subscriptionService = mock(SubscriptionService.class);
        when(this.subscriptionFactory.create(any())).thenReturn(subscriptionService);
        HttpResponse response = new HttpResponse();
        when(subscriptionService.query(any())).thenThrow(new SubscriptionException("error", response));
        this.sut.getSubscriptionFromCache(NOTIFICATION_ID);
        fail("should throw AppException");
    }

    @Test
    public void should_return200_whenSubscriptionGotFromRegistration() throws Exception {
        when(this.subscriptionCacheFactory.get(any())).thenReturn(null);
        ISubscriptionService subscriptionService = mock(SubscriptionService.class);
        when(this.subscriptionFactory.create(any())).thenReturn(subscriptionService);
        sut.setObjectMapper(new ObjectMapper());
        String jsonSubscription = this.getHmacSubscription();
        ObjectMapper objectMapper = new ObjectMapper();
        Subscription subscription = objectMapper.readValue(jsonSubscription, Subscription.class);
        List<Subscription> queryResult = new ArrayList<>();
        queryResult.add(subscription);
        when(subscriptionService.query(any())).thenReturn(queryResult);
        Subscription response = this.sut.getSubscriptionFromCache(NOTIFICATION_ID);
        Assert.assertEquals("testSubscription", response.getName());
    }

    private String getHmacSubscription() {
        return "{\n" +
                "\t\"name\": \"testSubscription\",\n" +
                "\t\"description\": \"Description\",\n" +
                "\t\"topic\": \"records-changed\",\n" +
                "\t\"pushEndpoint\": \"http://challenge\",\n" +
                "\t\"notificationId\": \"notificationId\",\n" +
                "\t\"id\": \"id_1\",\n" +
                "\t\"createdBy\": \"test@test.com\",\n" +
                "\t\"secret\": {\n" +
                "\t\t\"secretType\": \"HMAC\",\n" +
                "\t\t\"value\": \"testsecret\"\n" +
                "\t}\n" +
                "}";
    }
}
